@@ -1,6 +1,5 @@
 import json
 import sys
-import traceback
 
 from _pydev_bundle.pydev_is_thread_alive import is_thread_alive
 from _pydev_imps._pydev_saved_modules import thread
@@ -27,6 +26,7 @@ from pydevd_file_utils import get_abs_path_real_path_and_base_from_frame
 import pydevd_file_utils
 from pydevd_tracing import get_exception_traceback_str
 from _pydev_bundle._pydev_completer import completions_to_xml
+from _pydev_bundle import pydev_log
 
 if IS_IRONPYTHON:
 
@@ -38,7 +38,7 @@ if IS_IRONPYTHON:
 #=======================================================================================================================
 # NetCommandFactory
 #=======================================================================================================================
-class NetCommandFactory:
+class NetCommandFactory(object):
 
     def _thread_to_xml(self, thread):
         """ thread information as XML """
@@ -73,7 +73,7 @@ class NetCommandFactory:
         frame_description = pydevd_xml.make_valid_xml_value(frame_description)
         return NetCommand(CMD_THREAD_CREATE, 0, '<xml><thread name="%s" id="%s"/></xml>' % (frame_description, frame_id))
 
-    def make_list_threads_message(self, seq):
+    def make_list_threads_message(self, py_db, seq):
         """ returns thread listing as XML """
         try:
             threads = get_non_pydevd_threads()
@@ -87,11 +87,11 @@ class NetCommandFactory:
         except:
             return self.make_error_message(seq, get_exception_traceback_str())
 
-    def make_get_thread_stack_message(self, py_db, seq, thread_id, topmost_frame, must_be_suspended=False):
+    def make_get_thread_stack_message(self, py_db, seq, thread_id, topmost_frame, fmt, must_be_suspended=False, start_frame=0, levels=0):
         """
         Returns thread stack as XML.
 
-        :param be_suspended: If True and the thread is not suspended, returns None.
+        :param must_be_suspended: If True and the thread is not suspended, returns None.
         """
         try:
             # If frame is None, the return is an empty frame list.
@@ -146,12 +146,36 @@ class NetCommandFactory:
         except:
             return self.make_error_message(seq, get_exception_traceback_str())
 
-    def make_thread_killed_message(self, id):
+    def make_thread_killed_message(self, tid):
         try:
-            return NetCommand(CMD_THREAD_KILL, 0, str(id))
+            return NetCommand(CMD_THREAD_KILL, 0, str(tid))
         except:
             return self.make_error_message(0, get_exception_traceback_str())
 
+    def _iter_visible_frames_info(self, py_db, frame, frame_id_to_lineno):
+        while frame is not None:
+            if frame.f_code is None:
+                continue  # IronPython sometimes does not have it!
+
+            method_name = frame.f_code.co_name  # method name (if in method) or ? if global
+            if method_name is None:
+                continue  # IronPython sometimes does not have it!
+
+            abs_path_real_path_and_base = get_abs_path_real_path_and_base_from_frame(frame)
+            if py_db.get_file_type(abs_path_real_path_and_base) == py_db.PYDEV_FILE:
+                # Skip pydevd files.
+                frame = frame.f_back
+                continue
+
+            filename_in_utf8 = pydevd_file_utils.norm_file_to_client(abs_path_real_path_and_base[0])
+
+            frame_id = id(frame)
+            lineno = frame_id_to_lineno.get(frame_id, frame.f_lineno)
+
+            yield frame_id, frame, method_name, abs_path_real_path_and_base[0], filename_in_utf8, lineno
+
+            frame = frame.f_back
+
     def make_thread_stack_str(self, frame, frame_id_to_lineno=None):
         '''
         :param frame_id_to_lineno:
@@ -169,40 +193,19 @@ class NetCommandFactory:
         frame = None  # Clear frame reference
         try:
             py_db = get_global_debugger()
-            while curr_frame:
-                frame_id = id(curr_frame)
-
-                if curr_frame.f_code is None:
-                    break  # Iron Python sometimes does not have it!
-
-                method_name = curr_frame.f_code.co_name  # method name (if in method) or ? if global
-                if method_name is None:
-                    break  # Iron Python sometimes does not have it!
-
-                abs_path_real_path_and_base = get_abs_path_real_path_and_base_from_frame(curr_frame)
-                if py_db.get_file_type(abs_path_real_path_and_base) == py_db.PYDEV_FILE:
-                    # Skip pydevd files.
-                    curr_frame = curr_frame.f_back
-                    continue
-
-                filename_in_utf8 = pydevd_file_utils.norm_file_to_client(abs_path_real_path_and_base[0])
-                if not filesystem_encoding_is_utf8 and hasattr(filename_in_utf8, "decode"):
-                    # filename_in_utf8 is a byte string encoded using the file system encoding
-                    # convert it to utf8
-                    filename_in_utf8 = filename_in_utf8.decode(file_system_encoding).encode("utf-8")
+            for frame_id, frame, method_name, _original_filename, filename_in_utf8, lineno in self._iter_visible_frames_info(
+                    py_db, curr_frame, frame_id_to_lineno
+                ):
 
                 # print("file is ", filename_in_utf8)
-
-                lineno = frame_id_to_lineno.get(frame_id, curr_frame.f_lineno)
                 # print("line is ", lineno)
 
                 # Note: variables are all gotten 'on-demand'.
                 append('<frame id="%s" name="%s" ' % (frame_id , make_valid_xml_value(method_name)))
                 append('file="%s" line="%s">' % (quote(make_valid_xml_value(filename_in_utf8), '/>_= \t'), lineno))
                 append("</frame>")
-                curr_frame = curr_frame.f_back
         except:
-            traceback.print_exc()
+            pydev_log.exception()
 
         curr_frame = None  # Clear frame reference
         return ''.join(cmd_text_list)
@@ -270,7 +273,7 @@ class NetCommandFactory:
         except:
             return self.make_error_message(0, get_exception_traceback_str())
 
-    def make_thread_suspend_single_notification(self, thread_id, stop_reason):
+    def make_thread_suspend_single_notification(self, py_db, thread_id, stop_reason):
         try:
             return NetCommand(CMD_THREAD_SUSPEND_SINGLE_NOTIFICATION, 0, json.dumps(
                 {'thread_id': thread_id, 'stop_reason':stop_reason}))
